<?php

namespace App\Console\Commands;

use App\Repositories\SyncRepository;
use App\Sdk\SdkManager;

class SyncUserWithServer extends BaseCommand
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'sync:run';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Sync user data on server';

    /**
     * Execute the console command.
     *
     * @param SyncRepository $syncRepository
     * @return mixed
     * @throws \Prettus\Validator\Exceptions\ValidatorException
     */
    public function handle(SyncRepository $syncRepository)
    {
        $tasks = $syncRepository->all();

        foreach ($tasks as $task) {
            $user = $this->usersRepo->findWithoutFail($task->users_id);

            if (!$user) {
                $this->error('Не найден пользователь с ид ' . $task->users_id);
                continue;
            }

            $server = $this->serversRepo->findWithoutFail($user->servers_id);

            if (!$server) {
                $this->error('Не найден сервер для пользователя ' . $user->username);
                continue;
            }

            $sdk = (new SdkManager())->instance($user->server);

            if ($sdk === null) {
                $this->error('Не найден SDK сервера');
                continue;
            }

            $sdk->setBaseApiUrl();

            if (!$response = $sdk->login(['username' => $user->username, 'password' => decrypt($user->danger_password)])) {
                $this->error('Ошибка авторизации SDK сервера');
                continue;
            }

            // Ищем водителей
            $drivers = $sdk->getDrivers($response->user->gd, $user->username);

            foreach ((array)$drivers['data'] as $k => $driver) {
                list($surname, $name, $middlename) = array_pad(explode(' ', $driver['name'], 3), 3, null);

                $tmp = [
                    'surname'    => $surname,
                    'name'       => $name,
                    'middlename' => $middlename,
                    'users_id'   => $user->id,
                ];

                $this->driversRepo->updateOrCreate($tmp, $tmp);
            }

            // Ищем объекты
            $objects = $sdk->getUnits();

            foreach ((array)$objects['data'] as $object) {
                $tmp = [
                    'name'       => $object['name'],
                    'mon_id'     => $object['id'],
                    'servers_id' => $server->id,
                    'type'       => 'car',
                ];

                $this->objectsRepo->updateOrCreate(['mon_id' => $object['id'], 'users_id'   => $user->id], $tmp);
            }

            $syncRepository->delete($task->id);
        }

        return true;
    }
}
