<?php

namespace App\Console\Commands;

use App\Sdk\SdkManager;

class WaybillCreateesm7 extends BaseCommand
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'waybill:create:esm7 {id}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Создание документа ЭСМ - 7';

    /**
     * Execute the console command.
     *
     * @return mixed
     * @throws \PHPExcel_Exception
     */
    public function handle()
    {
        $waybill = $this->waybillsRepo->findWithoutFail($this->argument('id'));

        if (!$waybill) {
            $this->error('Не найден путевой лист');

            return self::$EXIT_CODE['NOT_FOUND'];
        }

        $start_date = $waybill->start_date;
        $end_date = $waybill->end_date;

        $sdk = (new SdkManager())->instance($waybill->user->server);

        if ($sdk === null) {
            $this->error('Не найден SDK сервера');

            return self::$EXIT_CODE['NOT_FOUND'];
        }

        $sdk->setBaseApiUrl();

        if (!$response = $sdk->login(['username' => $waybill->user->username, 'password' => decrypt($waybill->user->danger_password)])) {
            $this->error('Ошибка авторизации SDK сервера');

            return self::$EXIT_CODE['FAILED'];
        }

        $reportTrip = $sdk->getReport($response->user->gd);

        if (count($reportTrip) === 0) {
            $this->error('Не найден шаблон отчета');

            return self::$EXIT_CODE['FAILED'];
        }

        $unit = $sdk->getUnit($waybill->object->mon_id);

        if ($unit['data']['id'] === 0) {
            $this->error('Объект не найден на сервере');

            return self::$EXIT_CODE['NOT_FOUND'];
        }

        $organisation = '';
        $gos_num = $waybill->object->gos_number;
        $auto_mark = $waybill->object->model;

        if (count($unit['data']['fields']) > 0) {
            foreach ($unit['data']['fields'] as $custom_field) {
                $custom_field_name = $custom_field->nm;
                $custom_field_value = $custom_field->vl;

                if (empty($gos_num) && $custom_field_name === 'Государственный номерной знак') {
                    $gos_num = $custom_field_value;
                } elseif (empty($auto_mark) && $custom_field_name === 'Марка автомобиля') {
                    $auto_mark = $custom_field_value;
                } elseif (empty($organisation) && $custom_field_name === 'Организация') {
                    $organisation = $custom_field_value;
                }
            }
        }

        if (isset($waybill->owner) && !empty($waybill->owner)) {
            if ($waybill->owner->type === 'company') {
                $organisation = implode(' ', [$waybill->owner->company, ',', $waybill->owner->address, ',', $waybill->owner->phone]);
            } else {
                $organisation = implode(' ', [$waybill->owner->surname, $waybill->owner->name, $waybill->owner->middlename, ',', $waybill->owner->address, ',', $waybill->owner->phone]);
            }
        }

        $execReport = $sdk->execReport($reportTrip, $waybill->object->mon_id, $start_date, $end_date);

        if (!$execReport) {
            $this->error('Ошибка выполнения отчета');

            return self::$EXIT_CODE['FAILED'];
        }

        $tables = $sdk->getTablesReport();
        $stats = $sdk->getStatsReport();

        if (count($tables) === 0) {
            $this->error('Данные не получены');

            return self::$EXIT_CODE['FAILED'];
        }

        $machine_hours = 0;

        foreach ($stats as $stat) {
            list($stat_name, $stat_value) = (array)$stat;

            if ($stat_name === 'Моточасы') {
                $machine_hours = $stat_value;
            }
        }

        try {
            $objReader = \PHPExcel_IOFactory::createReader('Excel5');
            $objPHPExcel = $objReader->load(resource_path('/waytemplate/esm7.xls'));
            $objPHPExcel->setActiveSheetIndex();
            $aSheet = $objPHPExcel->getActiveSheet();
        } catch (\PHPExcel_Writer_Exception $e) {
            $this->error('Ошибка открытия excel шаблона');

            return self::$EXIT_CODE['FAILED'];
        } catch (\PHPExcel_Exception $e) {
            $this->error('Ошибка excel шаблона');

            return self::$EXIT_CODE['FAILED'];
        }

        $driver_name = $out_date = $arrival_date = '';

        foreach ($tables as $table_index => $table) {
            if ($table->nm === 'unit_trips') {
                $headers = (array)$table->h;
                $headers_assoc = [];

                $header_num = 0;
                foreach ($headers as $header) {
                    $headers_assoc[$header] = $header_num;

                    $header_num++;
                }

                $params = [
                    'tableIndex' => $table_index,
                    'indexFrom'  => 0,
                    'indexTo'    => $table->r,
                ];

                $response = $sdk->call('report/get_result_rows', $params);
                $trips = $response->body;

                $trip_num = 1;
                foreach ($trips as $trip) {
                    $ctrip_data_arr = (array)$trip->c;

                    foreach ($ctrip_data_arr as $data_num => $ctrip_data) {
                        $value = is_object($ctrip_data) ? $ctrip_data->{'t'} : $ctrip_data;

                        if ($trip_num === count($trips)) {
                            if ($data_num === $headers_assoc['Водитель']) {
                                $driver_name = $value;
                            }
                        }

                        ///////// ??????
                        if ($data_num === $headers_assoc['Водитель']) {
                            continue;
                        }
                    }

                    $trip_num++;
                }
            }
        }

        $aSheet->setCellValue('CK9', $start_date->day);
        $aSheet->setCellValue('CQ9', $start_date->month);
        $aSheet->setCellValue('CW9', $start_date->year);

        if (!empty($organisation)) {
            $aSheet->setCellValue('N10', $organisation);
        }

        if ($waybill->customer) {
            $aSheet->setCellValue('J12', $waybill->customer);
        }

        if ($waybill->place_work) {
            $aSheet->setCellValue('I14', $waybill->place_work);
        }

        $aSheet->setCellValue('J16', $waybill->object->name);

        if (!empty($auto_mark)) {
            $aSheet->setCellValue('BF16', $auto_mark);
        }

        $aSheet->setCellValue('CK16', $start_date->format('d.m.y H:i'));
        $aSheet->setCellValue('CT16', $end_date->format('d.m.y H:i'));
        $aSheet->getStyle('CK16')->getAlignment()->setWrapText(true);
        $aSheet->getStyle('CT16')->getAlignment()->setWrapText(true);

        if (!empty($gos_num)) {
            $aSheet->setCellValue('AH18', $gos_num);
        }

        if (isset($waybill->driver) && !empty($waybill->driver)) {
            $driver_name = implode(' ', [$waybill->driver->surname, $waybill->driver->name, $waybill->driver->middlename]);
        }

        $aSheet->setCellValue('M19', $driver_name);

        if ($waybill->type_work) {
            $aSheet->setCellValue('A26', $waybill->type_work);
        }

        if (!empty($machine_hours)) {
            $aSheet->setCellValue('BD26', $machine_hours);
            $aSheet->setCellValue('BD33', $machine_hours);

            $aSheet->setCellValue('H39', machineHoursPluralize($machine_hours));
        }

        try {
            $objWriter = \PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');

            $path = 'waybills/' . $waybill->object->name;

            if (!@mkdir(resource_path($path), 0700, true) && !is_dir(resource_path($path))) {
                $this->error('Ошибка создания папки для хранения excel документов');

                return self::$EXIT_CODE['FAILED'];
            }

            $file = $path . '/ЭСМ - 7 № ' . $waybill->id . '.xls';

            $objWriter->save(resource_path($file));

            $this->waybillsRepo->update(['file' => $file], $waybill->id);

            return self::$EXIT_CODE['SUCCESS'];
        } catch (\PHPExcel_Writer_Exception $e) {
            $this->error('Ошибка сохранения excel документа');

            return self::$EXIT_CODE['FAILED'];
        }
    }
}
