<?php

namespace App\Console\Commands;

use App\Sdk\SdkManager;
use Carbon\Carbon;

class WaybillCreatef1 extends BaseCommand
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'waybill:create:f1 {id}';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Создание путевого листа по форме №1';

    /**
     * Execute the console command.
     *
     * @return mixed
     * @throws \PHPExcel_Exception
     */
    public function handle()
    {
        $waybill = $this->waybillsRepo->findWithoutFail($this->argument('id'));

        if (!$waybill) {
            $this->error('Не найден путевой лист');

            return self::$EXIT_CODE['NOT_FOUND'];
        }

        $start_date = $waybill->start_date;
        $end_date = $waybill->end_date;

        $sdk = (new SdkManager())->instance($waybill->user->server);

        if ($sdk === null) {
            $this->error('Не найден SDK сервера');

            return self::$EXIT_CODE['NOT_FOUND'];
        }

        $sdk->setBaseApiUrl();

        if (!$response = $sdk->login(['username' => $waybill->user->username, 'password' => decrypt($waybill->user->danger_password)])) {
            $this->error('Ошибка авторизации SDK сервера');

            return self::$EXIT_CODE['FAILED'];
        }

        $reportTrip = $sdk->getReport($response->user->gd);
        $drivers = $sdk->getDrivers($response->user->gd, $waybill->user->username);

        if (count($reportTrip) === 0) {
            $this->error('Не найден шаблон отчета');

            return self::$EXIT_CODE['FAILED'];
        }

        $unit = $sdk->getUnit($waybill->object->mon_id);

        if ($unit['data']['id'] === 0) {
            $this->error('Объект не найден на сервере');

            return self::$EXIT_CODE['NOT_FOUND'];
        }

        $organisation = '';
        $garage_num = $waybill->object->garage_number;
        $gos_num = $waybill->object->gos_number;
        $auto_mark = $waybill->object->model;
        $fuel_mark = $waybill->object->fuel;

        if (count($unit['data']['fields']) > 0) {
            foreach ($unit['data']['fields'] as $custom_field) {
                $custom_field_name = $custom_field->nm;
                $custom_field_value = $custom_field->vl;

                if (empty($garage_num) && $custom_field_name === 'Гаражный номер') {
                    $garage_num = $custom_field_value;
                } elseif (empty($gos_num) && $custom_field_name === 'Государственный номерной знак') {
                    $gos_num = $custom_field_value;
                } elseif (empty($auto_mark) && $custom_field_name === 'Марка автомобиля') {
                    $auto_mark = $custom_field_value;
                } elseif (empty($fuel_mark) && $custom_field_name === 'Марка горючего') {
                    $fuel_mark = $custom_field_value;
                } elseif (empty($organisation) && $custom_field_name === 'Организация') {
                    $organisation = $custom_field_value;
                }
            }
        }

        if (isset($waybill->owner) && !empty($waybill->owner)) {
            if ($waybill->owner->type === 'company') {
                $organisation = implode(' ', [$waybill->owner->company, ',', $waybill->owner->address, ',', $waybill->owner->phone]);
            } else {
                $organisation = implode(' ', [$waybill->owner->surname, $waybill->owner->name, $waybill->owner->middlename, ',', $waybill->owner->address, ',', $waybill->owner->phone]);
            }
        }

        $execReport = $sdk->execReport($reportTrip, $waybill->object->mon_id, $start_date, $end_date);

        if (!$execReport) {
            $this->error('Ошибка выполнения отчета');

            return self::$EXIT_CODE['FAILED'];
        }

        $tables = $sdk->getTablesReport();
        $stats = $sdk->getStatsReport();

        if (count($tables) === 0) {
            $this->error('Данные не получены');

            return self::$EXIT_CODE['FAILED'];
        }

        $spent_fact = $spent_norm = '';
        $distance = $distance_all = 0;
        $fuel_start = $fuel_end = $fuel_all = $machine_hours = $tucked = 0;

        foreach ($stats as $stat) {
            list($stat_name, $stat_value) = (array)$stat;

            if ($stat_name === 'Потрачено по нормам в поездках') {
                $spent_norm = (float)$stat_value;
            } elseif ($stat_name === 'Потрачено по расчету в поездках') {
                $spent_fact = (float)$stat_value;
            } elseif ($stat_name === 'Счетчик пробега') {
                $distance_all = (float)$stat_value;
            } elseif ($stat_name === 'Пробег по всем сообщениям') {
                $distance = (float)$stat_value;
            } elseif ($stat_name === 'Нач. уровень') {
                $fuel_start = (float)$stat_value;
            } elseif ($stat_name === 'Конеч. уровень') {
                $fuel_end = (float)$stat_value;
            } elseif ($stat_name === 'Моточасы') {
                $machine_hours = $stat_value;
            } elseif ($stat_name === 'Всего заправлено') {
                $tucked = (float)$stat_value;
            }
        }

        try {
            $objReader = \PHPExcel_IOFactory::createReader('Excel5');
            $objPHPExcel = $objReader->load(resource_path('/waytemplate/f1.xls'));
            $objPHPExcel->setActiveSheetIndex(1);
            $aSheet = $objPHPExcel->getActiveSheet();
        } catch (\PHPExcel_Writer_Exception $e) {
            $this->error('Ошибка открытия excel шаблона');

            return self::$EXIT_CODE['FAILED'];
        } catch (\PHPExcel_Exception $e) {
            $this->error('Ошибка excel шаблона');

            return self::$EXIT_CODE['FAILED'];
        }

        $driver_name = $driver_table_number = $out_date = $arrival_date = '';

        foreach ($tables as $table_index => $table) {
            if ($table->nm === 'unit_trips') {
                $headers = (array)$table->h;
                $headers_assoc = [];

                $header_num = 0;
                foreach ($headers as $header) {
                    $headers_assoc[$header] = $header_num;

                    $header_num++;
                }

                $params = [
                    'tableIndex' => $table_index,
                    'indexFrom'  => 0,
                    'indexTo'    => $table->r,
                ];

                $response = $sdk->call('report/get_result_rows', $params);
                $trips = $response->body;

                $row = 4;
                $trip_num = 1;
                foreach ($trips as $trip) {
                    $ctrip_data_arr = (array)$trip->c;
                    $col = 'A';
//                    $col = 'B';
//                    $aSheet->setCellValue('A'.$row, $row - 3);

                    foreach ($ctrip_data_arr as $data_num => $ctrip_data) {
                        $value = is_object($ctrip_data) ? $ctrip_data->{'t'} : $ctrip_data;

                        if ($trip_num === 1) {
                            if ($data_num === $headers_assoc['Начало']) {
                                $out_date = new Carbon($value);
                            } elseif ($data_num === $headers_assoc['Нач. положение']) {
                                $trip_begin_addr = $value;
                            }
                        }

                        if ($trip_num === count($trips)) {
                            if ($data_num === $headers_assoc['Конец']) {
                                $arrival_date = new Carbon($value);
                            } elseif ($data_num === $headers_assoc['Конеч. счетчик']) {
                                $speed_km = $value;
                            } elseif ($data_num === $headers_assoc['Водитель']) {
                                $driver_name = $value;
                            }
                        }

                        ///////// ??????
                        if ($data_num === $headers_assoc['Водитель']) {
                            continue;
                        }

                        $aSheet->setCellValue($col.$row, $value);

                        $col++;
                    }

                    $row++;
                    $trip_num++;
                }

                $aSheet->setCellValue('F101', $distance  . ' km');
            }
        }

        $objPHPExcel->setActiveSheetIndex();
        $aSheet = $objPHPExcel->getActiveSheet();

        $aSheet->setCellValue('G5', $start_date->day);
        $aSheet->setCellValue('L5', getRussianNameMonth($start_date->month, 'я'));
        $aSheet->setCellValue('W5', $start_date->year);

        $aSheet->setCellValue('AN5', $end_date->day);
        $aSheet->setCellValue('AT5', getRussianNameMonth($end_date->month, 'я'));
        $aSheet->setCellValue('BE5', $end_date->year);

        /*Single fields data*/
        if (!empty($fuel_start)) {
            $aSheet->setCellValue('BZ28', $fuel_start);
        }

        if (!empty($fuel_end)) {
            $aSheet->setCellValue('BZ29', $fuel_end);
        }

        if (!empty($trip_begin_addr)) {
            $aSheet->setCellValue('T16', $trip_begin_addr);
        }

        if (!empty($out_date)) {
            $aSheet->setCellValue('AD21', $out_date);
        }

        if (!empty($arrival_date)) {
            $aSheet->setCellValue('AH24', $arrival_date);
        }

        if (!empty($speed_km)) {
            $aSheet->setCellValue('BZ35', $speed_km);
        }


        if (!empty($garage_num)) {
            $aSheet->setCellValue('BT11', $garage_num);
        }

        if (!empty($organisation)) {
            $aSheet->setCellValue('Q8', $organisation);
        }


        if (!empty($gos_num)) {
            $aSheet->setCellValue('AK11', $gos_num);
        }

        if (!empty($auto_mark)) {
            $aSheet->setCellValue('W10', $auto_mark);
        }

        if (!empty($fuel_mark)) {
            $aSheet->setCellValue('BL21', $fuel_mark);
        }

        if (!empty($driver_name)) {
            if (count($drivers) > 0) {
                foreach ($drivers['data'] as $driver_num => $driver_data) {
                    if ($driver_data['name'] === $driver_name) {
                        if (!empty($driver_data['table_number'])) {
                            $driver_table_number = $driver_data['table_number'];
                        }
                    }
                }
            }
        }

        if (isset($waybill->driver) && !empty($waybill->driver)) {
            $driver_name = implode(' ', [$waybill->driver->surname, $waybill->driver->name, $waybill->driver->middlename]);
            $driver_table_number = $waybill->driver->table_number;
        }

        $aSheet->setCellValue('N12', $driver_name);
        $aSheet->setCellValue('BT12', $driver_table_number);

        if (!empty($spent_norm)) {
            $aSheet->setCellValue('BZ30', $spent_norm);
        }

        if (!empty($spent_fact)) {
            $aSheet->setCellValue('BZ31', $spent_fact);
        }

        try {
            $attributes = [
                'fuel_top'     => (int)$fuel_start,
                'fuel_return'  => (int)$fuel_end,
                'tucked'       => (int)$tucked,
                'spent'        => (int)$spent_norm,
                'mileage'      => (int)$distance,
            ];

            if (!empty($out_date)) {
                $attributes['out_date'] = $out_date;
            }

            if (!empty($arrival_date)) {
                $attributes['arrival_date'] = $arrival_date;
            }

            $objWriter = \PHPExcel_IOFactory::createWriter($objPHPExcel, 'Excel2007');

            $path = 'waybills/' . $waybill->object->name;

            if (!@mkdir(resource_path($path), 0700, true) && !is_dir(resource_path($path))) {
                $this->error('Ошибка создания папки для хранения excel документов');

                return self::$EXIT_CODE['FAILED'];
            }

            $file = $path . '/путевой лист № ' . $waybill->id . '.xls';

            $objWriter->save(resource_path($file));

            $attributes['file'] = $file;

            $this->waybillsRepo->update($attributes, $waybill->id);

            return self::$EXIT_CODE['SUCCESS'];
        } catch (\PHPExcel_Writer_Exception $e) {
            $this->error('Ошибка сохранения excel документа');

            return self::$EXIT_CODE['FAILED'];
        }
    }
}
