<?php

namespace App\DataTables;

use App\Models\Owners;
use Form;
use Yajra\Datatables\Services\DataTable;

class OwnersDataTable extends DataTable
{

    /**
     * @return \Illuminate\Http\JsonResponse
     */
    public function ajax()
    {
        return $this->datatables
            ->eloquent($this->query())
            ->addColumn('action', 'owners.datatables_actions')
            ->make(true);
    }

    /**
     * Get the query object to be processed by datatables.
     *
     * @return \Illuminate\Database\Query\Builder|\Illuminate\Database\Eloquent\Builder
     */
    public function query()
    {
        $owners = Owners::query();

        if (!\Auth::user()->isAdmin()) {
            $owners = $owners->where('users_id', \Auth::user()->id);
        }

        return $this->applyScopes($owners);
    }

    /**
     * Optional method if you want to use html builder.
     *
     * @return \Yajra\Datatables\Html\Builder
     */
    public function html()
    {
        return $this->builder()
            ->columns($this->getColumns())
            ->addAction(['width' => '10%', 'title' => 'Управление'])
            ->ajax('')
            ->parameters([
                'dom'      => 'Bfrtip',
                'scrollX'  => false,
                'language' => [
                    'url' => url('vendor/datatables/Russian.json'),
                ],
                'buttons'  => [
                    'print',
                    'reset',
                    'reload',
                    [
                        'extend'  => 'collection',
                        'text'    => '<i class="fa fa-download"></i> Экспорт',
                        'buttons' => [
                            'csv',
                            'excel',
                            'pdf',
                        ],
                    ],
                    'colvis',
                ],
            ]);
    }

    /**
     * Get columns.
     *
     * @return array
     */
    private function getColumns()
    {
        return [
            'id'         => ['name' => 'id', 'data' => 'id', 'title' => 'Ид'],
            'type'       => ['name' => 'type', 'data' => 'type', 'title' => 'Тип'],
            'company'    => ['name' => 'company', 'data' => 'company', 'title' => 'Компания'],
            'surname'    => ['name' => 'surname', 'data' => 'surname', 'title' => 'Фамилия'],
            'name'       => ['name' => 'name', 'data' => 'name', 'title' => 'Имя'],
            'middlename' => ['name' => 'middlename', 'data' => 'middlename', 'title' => 'Отчество'],
            'address'    => ['name' => 'address', 'data' => 'address', 'title' => 'Адрес'],
            'phone'      => ['name' => 'phone', 'data' => 'phone', 'title' => 'Телефон'],
        ];
    }

    /**
     * Get filename for export.
     *
     * @return string
     */
    protected function filename()
    {
        return 'owners';
    }
}
