<?php

namespace App\Http\Controllers;

use App\DataTables\DriversDataTable;
use App\Http\Requests;
use App\Http\Requests\CreateDriversRequest;
use App\Http\Requests\UpdateDriversRequest;
use App\Repositories\DriversRepository;
use Flash;
use App\Http\Controllers\AppBaseController;
use Response;

class DriversController extends AppBaseController
{
    /** @var  DriversRepository */
    private $driversRepository;

    public function __construct(DriversRepository $driversRepo)
    {
        $this->driversRepository = $driversRepo;
    }

    /**
     * Display a listing of the Drivers.
     *
     * @param DriversDataTable $driversDataTable
     * @return Response
     */
    public function index(DriversDataTable $driversDataTable)
    {
        return $driversDataTable->render('drivers.index');
    }

    /**
     * Show the form for creating a new Drivers.
     *
     * @return Response
     */
    public function create()
    {
        return view('drivers.create');
    }

    /**
     * Store a newly created Drivers in storage.
     *
     * @param CreateDriversRequest $request
     *
     * @return Response
     */
    public function store(CreateDriversRequest $request)
    {
        $input = $request->all();

        $drivers = $this->driversRepository->create($input);

        Flash::success('Водитель успешно добавлен.');

        return redirect(route('drivers.index'));
    }

    /**
     * Display the specified Drivers.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function show($id)
    {
        $drivers = $this->driversRepository->findWithoutFail($id);

        if (empty($drivers)) {
            Flash::error('Водитель не найден');

            return redirect(route('drivers.index'));
        }

        return view('drivers.show')->with('drivers', $drivers);
    }

    /**
     * Show the form for editing the specified Drivers.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function edit($id)
    {
        $drivers = $this->driversRepository->findWithoutFail($id);

        if (empty($drivers)) {
            Flash::error('Водитель не найден');

            return redirect(route('drivers.index'));
        }

        return view('drivers.edit')->with('drivers', $drivers);
    }

    /**
     * Update the specified Drivers in storage.
     *
     * @param  int              $id
     * @param UpdateDriversRequest $request
     *
     * @return Response
     */
    public function update($id, UpdateDriversRequest $request)
    {
        $drivers = $this->driversRepository->findWithoutFail($id);

        if (empty($drivers)) {
            Flash::error('Водитель не найден');

            return redirect(route('drivers.index'));
        }

        $drivers = $this->driversRepository->update($request->all(), $id);

        Flash::success('Водитель успешно отредактирован.');

        return redirect(route('drivers.index'));
    }

    /**
     * Remove the specified Drivers from storage.
     *
     * @param  int $id
     *
     * @return Response
     */
    public function destroy($id)
    {
        $drivers = $this->driversRepository->findWithoutFail($id);

        if (empty($drivers)) {
            Flash::error('Водитель не найден');

            return redirect(route('drivers.index'));
        }

        $this->driversRepository->delete($id);

        Flash::success('Водитель успешно удален.');

        return redirect(route('drivers.index'));
    }
}
