<?php

namespace App\Models;

use Eloquent as Model;

/**
 * @SWG\Definition(
 *      definition="Objects",
 *      required={""},
 *      @SWG\Property(
 *          property="id",
 *          description="id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="name",
 *          description="name",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="gos_number",
 *          description="gos_number",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="type",
 *          description="type",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="mon_id",
 *          description="mon_id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="users_id",
 *          description="users_id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="servers_id",
 *          description="servers_id",
 *          type="integer",
 *          format="int32"
 *      )
 * )
 */
class Objects extends Model
{

    public $table = 'objects';

    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';


    public $fillable = [
        'name',
        'gos_number',
        'garage_number',
        'type',
        'mon_id',
        'users_id',
        'servers_id',
        'model',
        'fuel',
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id'         => 'integer',
        'name'       => 'string',
        'gos_number' => 'string',
        'type'       => 'string',
        'model'       => 'string',
        'fuel'       => 'string',
        'mon_id'     => 'integer',
        'users_id'   => 'integer',
        'servers_id' => 'integer',
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'type'       => 'required|in:car,truck,tractor,trailer',
        'name'       => 'required|string',
        'gos_number' => 'sometimes|nullable|string',
        'model' => 'sometimes|nullable|string',
        'fuel' => 'sometimes|nullable|string',
        'users_id'   => 'required|integer',
        'mon_id'     => 'sometimes|nullable|integer',
        'servers_id' => 'sometimes|nullable|integer',
    ];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function server()
    {
        return $this->belongsTo(\App\Models\Servers::class, 'servers_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\BelongsTo
     **/
    public function user()
    {
        return $this->belongsTo(\App\Models\Users::class, 'users_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function waybills()
    {
        return $this->hasMany(\App\Models\Waybills::class, 'objects_id', 'id');
    }
}
