<?php

namespace App\Models;

use Eloquent as Model;

/**
 * @SWG\Definition(
 *      definition="Servers",
 *      required={""},
 *      @SWG\Property(
 *          property="id",
 *          description="id",
 *          type="integer",
 *          format="int32"
 *      ),
 *      @SWG\Property(
 *          property="url",
 *          description="url",
 *          type="string"
 *      ),
 *      @SWG\Property(
 *          property="name",
 *          description="name",
 *          type="string"
 *      )
 *      @SWG\Property(
 *          property="type",
 *          description="type",
 *          type="string"
 *      )
 *      @SWG\Property(
 *          property="scheme",
 *          description="scheme",
 *          type="string"
 *      )
 *      @SWG\Property(
 *          property="port",
 *          description="port",
 *          type="integer"
 *      )
 * )
 *
 * @property string $url
 * @property string $name
 * @property string $scheme
 * @property integer $port
 */
class Servers extends Model
{

    public $table = 'servers';

    const CREATED_AT = 'created_at';
    const UPDATED_AT = 'updated_at';


    public $fillable = [
        'url',
        'name',
        'type',
        'scheme',
        'port',
    ];

    /**
     * The attributes that should be casted to native types.
     *
     * @var array
     */
    protected $casts = [
        'id'     => 'integer',
        'url'    => 'string',
        'name'   => 'string',
        'type'   => 'string',
        'scheme' => 'string',
        'port'   => 'integer',
    ];

    /**
     * Validation rules
     *
     * @var array
     */
    public static $rules = [
        'url'    => 'required|string',
        'name'   => 'required|string',
        'type'   => 'required|string|in:wialonpro,gelios',
        'scheme' => 'required|string|in:http,https',
        'port'   => 'sometimes|nullable|integer',
    ];

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function objects()
    {
        return $this->hasMany(\App\Models\Objects::class, 'servers_id', 'id');
    }

    /**
     * @return \Illuminate\Database\Eloquent\Relations\HasMany
     **/
    public function users()
    {
        return $this->hasMany(\App\Models\Users::class, 'servers_id', 'id');
    }
}
