<?php

namespace App\Sdk;

use Symfony\Component\Finder\Finder;

class SdkManager
{
    protected $mapper = [];
    protected $sdkTypesPath;

    /**
     * SdkManager constructor.
     *
     * @param string $path
     */
    public function __construct($path = '')
    {
        if (empty($path)) {
            $path = app_path('Sdk');
        }

        $this->setSdkTypesPath($path);
    }

    /**
     * @param $server
     * @return TypeSdkAbstract
     */
    public function instance($server)
    {
        $sdk = $this->getSdk($server);

        if (!$sdk) {
            $sdk = new NullSdk($server);
        }

        return $sdk;
    }

    /**
     * @param $server
     * @return bool
     */
    public function getSdk($server)
    {
        if (empty($this->getMapper())) {
            $files = $this->getFiles();

            foreach ($files as $file) {
                $ns = 'App\\Sdk';
                $name = $file->getBasename('.php');

                if ($relativePath = $file->getRelativePath()) {
                    $ns .= '\\' . str_replace('/', '\\', $relativePath);
                }

                $class = $ns . '\\' . $name;

                try {
                    $r = new \ReflectionClass($class);
                } catch (\ReflectionException $e) {
                    continue;
                }

                if ($r->isSubclassOf(TypeSdkAbstract::class) && !$r->isAbstract()) {
                    $obj = $r->newInstance();
                    $this->setMapper($obj->getName(), $obj);
                }
            }
        }

        if (!array_key_exists($server->type, $this->getMapper())) {
            return false;
        }

        return $this->getMapper($server->type)->setServer($server);
    }

    /**
     * @return array|Finder
     */
    public function getFiles()
    {
        try {
            $finder = new Finder();
            $finder->files()->name('*.php')->in($this->getSdkTypesPath());

            return $finder;
        } catch (\InvalidArgumentException $e) {
            return [];
        }
    }

    /**
     * @return mixed
     */
    public function getSdkTypesPath()
    {
        return $this->sdkTypesPath;
    }

    /**
     * @param mixed $sdkTypesPath
     */
    public function setSdkTypesPath($sdkTypesPath)
    {
        $this->sdkTypesPath = $sdkTypesPath;
    }

    /**
     * @param string $key
     * @return array
     */
    public function getMapper($key = '')
    {
        if (!empty($key)) {
            return $this->mapper[$key];
        }

        return $this->mapper;
    }

    /**
     * @param $name
     * @param $obj
     * @internal param array $mapper
     */
    public function setMapper($name, $obj)
    {
        $this->mapper[$name] = $obj;
    }
}