<?php

use App\Models\Servers;
use App\Repositories\ServersRepository;
use Illuminate\Foundation\Testing\DatabaseTransactions;

class ServersRepositoryTest extends TestCase
{
    use MakeServersTrait, ApiTestTrait, DatabaseTransactions;

    /**
     * @var ServersRepository
     */
    protected $serversRepo;

    public function setUp()
    {
        parent::setUp();
        $this->serversRepo = App::make(ServersRepository::class);
    }

    /**
     * @test create
     */
    public function testCreateServers()
    {
        $servers = $this->fakeServersData();
        $createdServers = $this->serversRepo->create($servers);
        $createdServers = $createdServers->toArray();
        $this->assertArrayHasKey('id', $createdServers);
        $this->assertNotNull($createdServers['id'], 'Created Servers must have id specified');
        $this->assertNotNull(Servers::find($createdServers['id']), 'Servers with given id must be in DB');
        $this->assertModelData($servers, $createdServers);
    }

    /**
     * @test read
     */
    public function testReadServers()
    {
        $servers = $this->makeServers();
        $dbServers = $this->serversRepo->find($servers->id);
        $dbServers = $dbServers->toArray();
        $this->assertModelData($servers->toArray(), $dbServers);
    }

    /**
     * @test update
     */
    public function testUpdateServers()
    {
        $servers = $this->makeServers();
        $fakeServers = $this->fakeServersData();
        $updatedServers = $this->serversRepo->update($fakeServers, $servers->id);
        $this->assertModelData($fakeServers, $updatedServers->toArray());
        $dbServers = $this->serversRepo->find($servers->id);
        $this->assertModelData($fakeServers, $dbServers->toArray());
    }

    /**
     * @test delete
     */
    public function testDeleteServers()
    {
        $servers = $this->makeServers();
        $resp = $this->serversRepo->delete($servers->id);
        $this->assertTrue($resp);
        $this->assertNull(Servers::find($servers->id), 'Servers should not exist in DB');
    }
}
