<?php

use App\Models\Users;
use App\Repositories\UsersRepository;
use Illuminate\Foundation\Testing\DatabaseTransactions;

class UsersRepositoryTest extends TestCase
{
    use MakeUsersTrait, ApiTestTrait, DatabaseTransactions;

    /**
     * @var UsersRepository
     */
    protected $usersRepo;

    public function setUp()
    {
        parent::setUp();
        $this->usersRepo = App::make(UsersRepository::class);
    }

    /**
     * @test create
     */
    public function testCreateUsers()
    {
        $users = $this->fakeUsersData();
        $createdUsers = $this->usersRepo->create($users);
        $createdUsers = $createdUsers->toArray();
        $this->assertArrayHasKey('id', $createdUsers);
        $this->assertNotNull($createdUsers['id'], 'Created Users must have id specified');
        $this->assertNotNull(Users::find($createdUsers['id']), 'Users with given id must be in DB');
        $this->assertModelData($users, $createdUsers);
    }

    /**
     * @test read
     */
    public function testReadUsers()
    {
        $users = $this->makeUsers();
        $dbUsers = $this->usersRepo->find($users->id);
        $dbUsers = $dbUsers->toArray();
        $this->assertModelData($users->toArray(), $dbUsers);
    }

    /**
     * @test update
     */
    public function testUpdateUsers()
    {
        $users = $this->makeUsers();
        $fakeUsers = $this->fakeUsersData();
        $updatedUsers = $this->usersRepo->update($fakeUsers, $users->id);
        $this->assertModelData($fakeUsers, $updatedUsers->toArray());
        $dbUsers = $this->usersRepo->find($users->id);
        $this->assertModelData($fakeUsers, $dbUsers->toArray());
    }

    /**
     * @test delete
     */
    public function testDeleteUsers()
    {
        $users = $this->makeUsers();
        $resp = $this->usersRepo->delete($users->id);
        $this->assertTrue($resp);
        $this->assertNull(Users::find($users->id), 'Users should not exist in DB');
    }
}
